@@ -20,6 +20,23 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
+#ifndef _GNU_SOURCE
+#error _GNU_SOURCE must be defined
+#endif
+#include <stdio.h>
+#include <stdbool.h>
+
+#define GNUC_UNUSED __attribute__((__unused__))
+
+static bool str_equal (const char *a, const char *b)
+{
+	if (a == NULL && b == NULL)
+		return true;
+	if (a == NULL || b == NULL)
+		return false;
+	return (strcmp (a, b) == 0);
+}
+
 struct {
 	const char *dbus_name;
 	const char *place_str_generic;
@@ -88,7 +105,7 @@ struct {
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wformat-nonliteral"
 
-static char *finger_str_to_msg(const char *finger_name, const char *driver_name, gboolean is_swipe)
+GNUC_UNUSED static char *finger_str_to_msg(const char *finger_name, const char *driver_name, bool is_swipe)
 {
 	int i;
 
@@ -96,17 +113,25 @@ static char *finger_str_to_msg(const char *finger_name, const char *driver_name,
 		return NULL;
 
 	for (i = 0; fingers[i].dbus_name != NULL; i++) {
-		if (g_str_equal (fingers[i].dbus_name, finger_name)) {
-			if (is_swipe == FALSE) {
-				if (driver_name)
-					return g_strdup_printf (TR (fingers[i].place_str_specific), driver_name);
-				else
-					return g_strdup (TR (fingers[i].place_str_generic));
+		if (str_equal (fingers[i].dbus_name, finger_name)) {
+			if (is_swipe == false) {
+				if (driver_name) {
+					char *s;
+					int ret;
+					ret = asprintf (&s, TR (fingers[i].place_str_specific), driver_name);
+					return ret >= 0 ? s : NULL;
+				} else {
+					return strdup (TR (fingers[i].place_str_generic));
+				}
 			} else {
-				if (driver_name)
-					return g_strdup_printf (TR (fingers[i].swipe_str_specific), driver_name);
-				else
-					return g_strdup (TR (fingers[i].swipe_str_generic));
+				if (driver_name) {
+					char *s;
+					int ret;
+					ret = asprintf (&s, TR (fingers[i].swipe_str_specific), driver_name);
+					return ret >= 0 ? s : NULL;
+				} else {
+					return strdup (TR (fingers[i].swipe_str_generic));
+				}
 			}
 		}
 	}
@@ -121,13 +146,13 @@ static char *finger_str_to_msg(const char *finger_name, const char *driver_name,
  * verify-match
  * verify-unknown-error
  */
-static const char *verify_result_str_to_msg(const char *result, gboolean is_swipe)
+GNUC_UNUSED static const char *verify_result_str_to_msg(const char *result, bool is_swipe)
 {
 	if (result == NULL)
 		return NULL;
 
 	if (strcmp (result, "verify-retry-scan") == 0) {
-		if (is_swipe == FALSE)
+		if (is_swipe == false)
 			return N_("Place your finger on the reader again");
 		else
 			return N_("Swipe your finger again");
@@ -147,13 +172,13 @@ static const char *verify_result_str_to_msg(const char *result, gboolean is_swip
  * enroll-failed
  * enroll-unknown-error
  */
-static const char *enroll_result_str_to_msg(const char *result, gboolean is_swipe)
+GNUC_UNUSED static const char *enroll_result_str_to_msg(const char *result, bool is_swipe)
 {
 	if (result == NULL)
 		return NULL;
 
 	if (strcmp (result, "enroll-retry-scan") == 0 || strcmp (result, "enroll-stage-passed") == 0) {
-		if (is_swipe == FALSE)
+		if (is_swipe == false)
 			return N_("Place your finger on the reader again");
 		else
 			return N_("Swipe your finger again");
