@@ -18,24 +18,22 @@
  *
  */
 
-#ifndef FILE_STORAGE_H
+#pragma once
 
-#define FILE_STORAGE_H
+int file_storage_print_data_save(FpPrint *print);
 
-int file_storage_print_data_save(struct fp_print_data *data,
-	enum fp_finger finger, const char *username);
+int file_storage_print_data_load(FpDevice *dev,
+                                 FpFinger  finger,
+                                 const char *username,
+                                 FpPrint **print);
 
-
-int file_storage_print_data_load(struct fp_dev *dev,
-	enum fp_finger finger, struct fp_print_data **data, const char *username);
-
-int file_storage_print_data_delete(struct fp_dscv_dev *dev,
-	enum fp_finger finger, const char *username);
+int file_storage_print_data_delete(FpDevice *dev,
+                                   FpFinger finger,
+                                   const char *username);
 
 int file_storage_init(void);
 
 int file_storage_deinit(void);
 
-GSList *file_storage_discover_prints(struct fp_dscv_dev *dev, const char *username);
-
-#endif
+GSList *file_storage_discover_prints(FpDevice *dev, const char *username);
+GSList *file_storage_discover_users(void);
